"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGlobalConfigPath = exports.writeRawGlobalSettings = exports.getRawGlobalSettings = void 0;
const logger_1 = require("../util/logger");
const configstore_1 = __importDefault(require("configstore"));
const packageName = 'cspell';
function getRawGlobalSettings() {
    const name = 'CSpell Configstore';
    const globalConf = {
        source: {
            name,
            filename: undefined,
        },
    };
    try {
        const cfgStore = new configstore_1.default(packageName);
        const cfg = cfgStore.all;
        // Only populate globalConf is there are values.
        if (cfg && Object.keys(cfg).length) {
            Object.assign(globalConf, cfg);
            globalConf.source = {
                name,
                filename: cfgStore.path,
            };
        }
    }
    catch (error) {
        logger_1.logError(error);
    }
    return globalConf;
}
exports.getRawGlobalSettings = getRawGlobalSettings;
function writeRawGlobalSettings(settings) {
    const toWrite = {
        import: settings.import,
    };
    try {
        const cfgStore = new configstore_1.default(packageName);
        cfgStore.set(toWrite);
        return undefined;
    }
    catch (error) {
        if (error instanceof Error)
            return error;
        return new Error(error.toString());
    }
}
exports.writeRawGlobalSettings = writeRawGlobalSettings;
function getGlobalConfigPath() {
    const cfgStore = new configstore_1.default(packageName);
    return cfgStore.path;
}
exports.getGlobalConfigPath = getGlobalConfigPath;
//# sourceMappingURL=GlobalSettings.js.map